searchNodes=[{"doc":"A behaviour module that defines both an Ecto schema and a JSON schema. An Injecto schema uses the module attribute @properties to define an Ecto schema and a JSON schema based on the ex_json_schema library. In doing so, it also injects a Jason encoder implementation. The advantage of using an Injecto schema is to get a consistent parsing and validating with Ecto changesets and JSON schema respectively with minimal boilerplates. This consistency is helpful when working with struct-based request or response bodies, because we can get accurate Swagger schemas for free. Example In the following documentation, we will use this simple Injecto schema as example: defmodule Post do @properties %{ title : { :string , required : true } , description : { :string , [ ] } , likes : { :integer , required : true , minimum : 0 } } use Injecto end The module attribute @properties must be defined first before invoking use Injecto . The properties attribute is a map with field names as keys and field specs as values. A field spec is a 2-tuple of {type, options} . For scalar types, most Ecto field types are supported, namely: [ :binary , :binary_id , :boolean , :float , :id , :integer , :string , :map , :decimal , :date , :time , :time_usec , :naive_datetime , :naive_datetime_usec , :utc_datetime , :utc_datetime_usec ] Refer to Ecto's documentation on Primitive Types to see how these field types get translated into Elixir types. Supported compound types include: {:enum, atoms} and {:enum, keyword} ; {:object, injecto_module} ; and {:array, inner_type} where inner_type can be a scalar, enum or object type. Usage: Ecto On the Ecto side, new/0 and changeset/2 functions can create a nil -filled struct and an Ecto changeset respectively. iex&gt; Post . new ( ) % Post { title : nil , description : nil , likes : nil } iex&gt; % Ecto.Changeset { valid? : false , errors : errors } = Post . changeset ( % Post { } , %{ } ) iex&gt; errors [ likes : { &quot;can&#39;t be blank&quot; , [ validation : :required ] } , title : { &quot;can&#39;t be blank&quot; , [ validation : :required ] } ] iex&gt; post = %{ title : &quot;Valid&quot; , likes : 10 } iex&gt; % Ecto.Changeset { valid? : true , errors : [ ] } = Post . changeset ( % Post { } , post ) The parse/2 function convert a map to a changeset-validated struct. iex&gt; { :error , errors } = Post . parse ( %{ } ) iex&gt; errors %{ likes : [ { &quot;can&#39;t be blank&quot; , [ validation : :required ] } ] , title : [ { &quot;can&#39;t be blank&quot; , [ validation : :required ] } ] } iex&gt; post = %{ title : &quot;Valid&quot; , likes : 10 } iex&gt; { :ok , % Post { title : &quot;Valid&quot; , likes : 10 } } = Post . parse ( post ) iex&gt; valid_posts = [ %{ title : &quot;A&quot; , likes : 1 } , %{ title : &quot;B&quot; , likes : 2 } ] iex&gt; { :ok , posts } = Post . parse_many ( valid_posts ) iex&gt; Enum . sort_by ( posts , &amp; ( &amp;1 . title ) ) [ % Post { title : &quot;A&quot; , likes : 1 , description : nil } , % Post { title : &quot;B&quot; , likes : 2 , description : nil } ] The parse_many/2 function is the collection counter part of parse/2 . One validation error is considered to be an error for the entire collection: iex&gt; invalid_posts = [ %{ title : 1 , likes : &quot;A&quot; } , %{ title : 2 , likes : &quot;B&quot; } ] iex&gt; { :error , errors } = Post . parse_many ( invalid_posts ) iex&gt; errors [ %{ likes : [ { &quot;is invalid&quot; , [ type : :integer , validation : :cast ] } ] , title : [ { &quot;is invalid&quot; , [ type : :string , validation : :cast ] } ] } , %{ likes : [ { &quot;is invalid&quot; , [ type : :integer , validation : :cast ] } ] , title : [ { &quot;is invalid&quot; , [ type : :string , validation : :cast ] } ] } ] iex&gt; valid_posts = [ %{ title : &quot;A&quot; , likes : 1 } , %{ title : &quot;B&quot; , likes : 2 } ] iex&gt; invalid_posts = [ %{ title : 1 , likes : &quot;A&quot; } ] iex&gt; { :error , errors } = Post . parse_many ( valid_posts ++ invalid_posts ) iex&gt; errors [ %{ likes : [ { &quot;is invalid&quot; , [ type : :integer , validation : :cast ] } ] , title : [ { &quot;is invalid&quot; , [ type : :string , validation : :cast ] } ] } ] Note that JSON schema constraints such as minimum: 0 are not caught by the Ecto changeset: iex&gt; post = %{ title : &quot;Invalid&quot; , likes : - 1 } iex&gt; % Ecto.Changeset { valid? : true , errors : [ ] } = Post . changeset ( % Post { } , post ) Usage: JSON Schema The function json_schema/0 returns a resolved ExJsonSchema.Scheam.Root struct. iex&gt; % ExJsonSchema.Schema.Root { schema : schema } = Post . json_schema ( ) iex&gt; schema %{ &quot;properties&quot; =&gt; %{ &quot;description&quot; =&gt; %{ &quot;anyOf&quot; =&gt; [ %{ &quot;type&quot; =&gt; &quot;string&quot; } , %{ &quot;type&quot; =&gt; &quot;null&quot; } ] } , &quot;likes&quot; =&gt; %{ &quot;minimum&quot; =&gt; 0 , &quot;type&quot; =&gt; &quot;integer&quot; } , &quot;title&quot; =&gt; %{ &quot;type&quot; =&gt; &quot;string&quot; } } , &quot;required&quot; =&gt; [ &quot;likes&quot; , &quot;title&quot; ] , &quot;title&quot; =&gt; &quot;Elixir.Post&quot; , &quot;type&quot; =&gt; &quot;object&quot; , &quot;x-struct&quot; =&gt; &quot;Elixir.Post&quot; } Internally, this is used by validate_json/1 to validate a map using the JSON schema. iex&gt; valid_post = %{ title : &quot;A&quot; , likes : 1 } iex&gt; { :ok , ^ valid_post } = Post . validate_json ( valid_post ) iex&gt; invalid_post = %{ title : 123 , likes : - 1 } iex&gt; { :error , errors } = Post . validate_json ( invalid_post ) iex&gt; Enum . sort ( errors ) [ { &quot;Expected the value to be &gt;= 0&quot; , &quot;#/likes&quot; } , { &quot;Type mismatch. Expected String but got Integer.&quot; , &quot;#/title&quot; } ]","ref":"Injecto.html","title":"Injecto","type":"behaviour"},{"doc":"Returns an Ecto changeset: iex&gt; % Ecto.Changeset { valid? : false , errors : errors } = Post . changeset ( % Post { } , %{ } ) iex&gt; errors [ likes : { &quot;can&#39;t be blank&quot; , [ validation : :required ] } , title : { &quot;can&#39;t be blank&quot; , [ validation : :required ] } ] iex&gt; post = %{ title : &quot;Valid&quot; , likes : 10 } iex&gt; % Ecto.Changeset { valid? : true , errors : [ ] } = Post . changeset ( % Post { } , post ) Note that JSON schema constraints such as minimum: 0 are not caught by the Ecto changeset: iex&gt; post = %{ title : &quot;Invalid&quot; , likes : - 1 } iex&gt; % Ecto.Changeset { valid? : true , errors : [ ] } = Post . changeset ( % Post { } , post )","ref":"Injecto.html#c:changeset/2","title":"Injecto.changeset/2","type":"callback"},{"doc":"Validates and returns an ex_json_schema schema: iex&gt; % ExJsonSchema.Schema.Root { schema : schema } = Post . json_schema ( ) iex&gt; schema %{ &quot;properties&quot; =&gt; %{ &quot;description&quot; =&gt; %{ &quot;anyOf&quot; =&gt; [ %{ &quot;type&quot; =&gt; &quot;string&quot; } , %{ &quot;type&quot; =&gt; &quot;null&quot; } ] } , &quot;likes&quot; =&gt; %{ &quot;minimum&quot; =&gt; 0 , &quot;type&quot; =&gt; &quot;integer&quot; } , &quot;title&quot; =&gt; %{ &quot;type&quot; =&gt; &quot;string&quot; } } , &quot;required&quot; =&gt; [ &quot;likes&quot; , &quot;title&quot; ] , &quot;title&quot; =&gt; &quot;Elixir.Post&quot; , &quot;type&quot; =&gt; &quot;object&quot; , &quot;x-struct&quot; =&gt; &quot;Elixir.Post&quot; }","ref":"Injecto.html#c:json_schema/0","title":"Injecto.json_schema/0","type":"callback"},{"doc":"Returns the struct with the fields populated with nil s: iex&gt; Post . new ( ) % Post { title : nil , description : nil , likes : nil }","ref":"Injecto.html#c:new/0","title":"Injecto.new/0","type":"callback"},{"doc":"Returns a result of a validated Elixir struct or the validation errors: iex&gt; { :error , errors } = Post . parse ( %{ } ) iex&gt; errors %{ likes : [ { &quot;can&#39;t be blank&quot; , [ validation : :required ] } ] , title : [ { &quot;can&#39;t be blank&quot; , [ validation : :required ] } ] } iex&gt; post = %{ title : &quot;Valid&quot; , likes : 10 } iex&gt; { :ok , % Post { title : &quot;Valid&quot; , likes : 10 } } = Post . parse ( post ) Note that JSON schema constraints such as minimum: 0 are not caught by parse/2 by default. Pass in the option :validate_json for JSON schema validation: iex&gt; post = %{ title : &quot;Invalid&quot; , likes : - 1 } iex&gt; { :ok , % Post { } } = Post . parse ( post ) iex&gt; post = %{ title : &quot;Invalid&quot; , likes : - 1 } iex&gt; { :error , errors } = Post . parse ( post , validate_json : true ) iex&gt; errors [ { &quot;Expected the value to be &gt;= 0&quot; , &quot;#/likes&quot; } ]","ref":"Injecto.html#c:parse/2","title":"Injecto.parse/2","type":"callback"},{"doc":"Calls parse/2 on a list of maps. Returns :ok if all maps are parsed correctly. iex&gt; valid_posts = [ %{ title : &quot;A&quot; , likes : 1 } , %{ title : &quot;B&quot; , likes : 2 } ] iex&gt; { :ok , posts } = Post . parse_many ( valid_posts ) iex&gt; Enum . sort_by ( posts , &amp; ( &amp;1 . title ) ) [ % Post { title : &quot;A&quot; , likes : 1 , description : nil } , % Post { title : &quot;B&quot; , likes : 2 , description : nil } ] iex&gt; invalid_posts = [ %{ title : 1 , likes : &quot;A&quot; } , %{ title : 2 , likes : &quot;B&quot; } ] iex&gt; { :error , errors } = Post . parse_many ( invalid_posts ) iex&gt; errors [ %{ likes : [ { &quot;is invalid&quot; , [ type : :integer , validation : :cast ] } ] , title : [ { &quot;is invalid&quot; , [ type : :string , validation : :cast ] } ] } , %{ likes : [ { &quot;is invalid&quot; , [ type : :integer , validation : :cast ] } ] , title : [ { &quot;is invalid&quot; , [ type : :string , validation : :cast ] } ] } ] iex&gt; valid_posts = [ %{ title : &quot;A&quot; , likes : 1 } , %{ title : &quot;B&quot; , likes : 2 } ] iex&gt; invalid_posts = [ %{ title : 1 , likes : &quot;A&quot; } ] iex&gt; { :error , errors } = Post . parse_many ( valid_posts ++ invalid_posts ) iex&gt; errors [ %{ likes : [ { &quot;is invalid&quot; , [ type : :integer , validation : :cast ] } ] , title : [ { &quot;is invalid&quot; , [ type : :string , validation : :cast ] } ] } ] Note that JSON schema constraints such as minimum: 0 are not caught by parse by default. Pass in the option :validate_json for JSON schema validation: iex&gt; posts = [ %{ title : &quot;A&quot; , likes : - 1 } ] iex&gt; { :ok , _ } = Post . parse_many ( posts ) iex&gt; { :error , errors } = Post . parse_many ( posts , validate_json : true ) iex&gt; errors [ [ { &quot;Expected the value to be &gt;= 0&quot; , &quot;#/likes&quot; } ] ]","ref":"Injecto.html#c:parse_many/2","title":"Injecto.parse_many/2","type":"callback"},{"doc":"Serialises a map, and validates the deserialised result against the JSON schema: iex&gt; valid_post = %{ title : &quot;A&quot; , likes : 1 } iex&gt; { :ok , ^ valid_post } = Post . validate_json ( valid_post ) iex&gt; invalid_post = %{ title : 123 , likes : - 1 } iex&gt; { :error , errors } = Post . validate_json ( invalid_post ) iex&gt; Enum . sort ( errors ) [ { &quot;Expected the value to be &gt;= 0&quot; , &quot;#/likes&quot; } , { &quot;Type mismatch. Expected String but got Integer.&quot; , &quot;#/title&quot; } ]","ref":"Injecto.html#c:validate_json/1","title":"Injecto.validate_json/1","type":"callback"}]